/***************************************************************************
;
;    This module is presented here only to serve as a sample for
;    8051 program development on BiPOM Electronics micro-controller boards.
;    BiPOM Electronics does not guarantee its functionality
;    with other micro-controller systems.
;    Please e-mail any questions and suggestions to info@bipom.com
;
;    Module:		ipxserv.c
;
;    Revision:		1.01
;
;    Description:	MM51E IPX server
;
;    This program is written using Micro-C Compiler from
;    Dunfield Development Systems.
;
****************************************************************************/
#include <8051io.h>
#include <8051bit.h>            /* Bit set/clear macros */
#include <8051reg.h>
#include <BipomLib\INCLUDE\types.h>
#include <BipomLib\MM51E\CS8900i\cs8900i.h>
#include <BipomLib\MM51E\CS8900i\cs8900ri.h>
#include <BipomLib\MM51E\IPX\ipx.h>

#define TX_SOCKET	0x6200
#define RX_SOCKET	0x6201

#define BUFER_NUMBER	0

char *serverReply = "IPX SERVER:";

void main()
{
	UINT txBuffer,rxBuffer;
	UINT ndx;
	UBYTE dataByte;
	ERRCODE ec;
	serinit(9600);		// Initialize UART
	printf("\nMM51E IPX SERVER");
	/* Set MAC address of my PC Ethernet card for now */
	OurMAC[0] = 0x00;
		OurMAC[1] = 0x00;
			OurMAC[2] = 0xE8;
				OurMAC[3] = 0x78;
					OurMAC[4] = 0x8E;
						OurMAC[5] = 0xDB;
//						
// Initialize the CS8900 layer						
//
	ec = CS8900_INIT();
	if (ec == SUCCESS)
		printf("\nCS8900 was initialized successfully");
	else
	{
		printf("\nERROR: Can't initialize CS8900");
		printf("\n!!! STOP !!!");
		for(;;);
	}
	//	
	//	Initialize IPX layer
	//
	/* destination network  = 00:00:00:00 */
	for (ndx = 0; ndx < IPX_NET_LEN; ndx++)
		OurIPX.DestNet[ndx]=0; 		
	/* destination MAC address  = FF:FF:FF:FF:FF:FF */	
	for (ndx = 0; ndx < ETHERNET_LEN; ndx++)
		OurIPX.DestMAC[ndx]=0xFF; 		
	/* destination socket */	
	OurIPX.DestSocket = TX_SOCKET;			 	
	/* source network */
	for (ndx = 0; ndx < IPX_NET_LEN; ndx++)
		OurIPX.SourceNet[ndx]=0; 
	/* source socket */			
	OurIPX.SourceSocket = RX_SOCKET;			 	
	IPX_Init();
	if (IPX_Valid == IPX_VALID)
		printf("\nIPX was initialized successfully");
	else
	{
		printf("\nERROR: Can't initialize IPX");
		printf("\n!!! STOP !!!");
		for(;;);
	}
//	
	for (;;)
	{
		printf("\n\nWait Ethernet frame...");
		while (CS8900_POLL() != PACKET_WAS_RECEIVED){}
		if (IPX_Receive() == SUCCESS)
		{
			// Prepare data 
			printf("\n\nReply to client");
	 		rxBuffer = IPX_GetRxDataPointer();
	 		txBuffer = IPX_GetTxDataPointer(BUFER_NUMBER);
	 		
	 		// Add SERVER message on entry
	 		for(ndx=0;;ndx++)
	 		{
	 			dataByte = *(serverReply+ndx);
	 			if(dataByte==0) break;
	 			IPX_SetData(txBuffer++,dataByte);	
	 		}
			// Copy HOST message to buffer 
			printf("\nHOST MESSAGE: ");
			for (ndx = 0; ndx< IPX_Length; ndx++)
			{
				dataByte = IPX_GetData(rxBuffer++);
				IPX_SetData(txBuffer++,dataByte);
				printf("%c",dataByte);
			}
		
			IPX_Send(BUFER_NUMBER,txBuffer-IPX_GetTxDataPointer(BUFER_NUMBER));
		}
		else
		{
			printf(".");	// Indicate that some Ethernet frame has been received 
		}
	}	
}
